import ffmpeg
from ffmpeg.nodes import FilterableStream
import os
from loguru import logger

RESOLUTION = (480, 852)
FRAMERATE = 30


class ConcatStories:
  def __init__(
    self,
    sorted_stories: list[str],
    output: str,
    loop_duration_image: int = 1,
    is_quiet: bool = True,
  ):
    self.stories = sorted_stories
    self.output = output
    self.resolution = RESOLUTION
    self.loop_duration_image = loop_duration_image
    self.is_quiet = is_quiet

  def _fix_aspect_ratio(self, stream: FilterableStream) -> FilterableStream:
    stream = ffmpeg.filter_(
      stream["v"],
      "scale",
      width=self.resolution[0],
      height=self.resolution[1],
      force_original_aspect_ratio="decrease",
    )
    stream = ffmpeg.filter_(
      stream,
      "pad",
      width=self.resolution[0],
      height=self.resolution[1],
      x="(ow-iw)/2",
      y="(oh-ih)/2",
      color="black",
    )
    return stream

  def concat(self) -> None:
    input_streams_spread = []

    for path in self.stories:
      if path.endswith(".mp4"):
        stream = ffmpeg.input(path)
        probe = ffmpeg.probe(path)

        stream_adjusted = self._fix_aspect_ratio(stream)
        if len(probe["streams"]) < 2:
          empty_audio = ffmpeg.input(
            "anullsrc", f="lavfi", t=probe["streams"][0]["duration"]
          )
          input_streams_spread.extend([stream_adjusted, empty_audio])
        else:
          input_streams_spread.extend([stream_adjusted, stream["a"]])
      else:
        stream = ffmpeg.input(
          path,
          t=self.loop_duration_image,
          loop=1,
          framerate=FRAMERATE,
        )
        stream = self._fix_aspect_ratio(stream)

        empty_audio = ffmpeg.input("anullsrc", f="lavfi", t=self.loop_duration_image)
        input_streams_spread.extend([stream, empty_audio])

    joined = ffmpeg.concat(*input_streams_spread, v=1, a=1, unsafe=True).node
    loglevel = "quiet" if self.is_quiet else "info"
    ffmpeg.output(
      joined[0], joined[1], os.path.join(".", self.output + ".mp4"), loglevel=loglevel
    ).run()
    logger.info(f"Stories concatenated to {self.output}.mp4")



import argparse
import subprocess
from loguru import logger
import sys
import shutil
from importlib.metadata import version

from concat_stories.snapchat_dl import SnapchatDL
from concat_stories.con_stories import ConcatStories


def is_ffmpeg_installed() -> bool:
  try:
    result = subprocess.run(
      ["ffmpeg", "-version"], stdout=subprocess.PIPE, stderr=subprocess.PIPE
    )
    return result.returncode == 0
  except FileNotFoundError:
    return False


def main():
  parser = argparse.ArgumentParser()
  parser.add_argument(
    "-u",
    "--username",
    help="Snapchat username ex. djkhaled305",
    type=str,
    required=True,
    dest="username",
  )
  parser.add_argument(
    "-o",
    "--output",
    help="Output video name ex. dj_khaled_stories",
    type=str,
    dest="output",
    metavar="OUTPUT_NAME",
  )
  parser.add_argument(
    "-d",
    "--delete",
    help="Delete stories after download.",
    action="store_true",
    default=False,
    dest="delete",
  )
  parser.add_argument(
    "-w",
    "--wait",
    help="Wait for user to delete unwanted stories.",
    action="store_true",
    default=False,
    dest="wait",
  )
  parser.add_argument(
    "-l",
    "--limit-story",
    help="Set maximum number of stories to download.",
    type=int,
    default=-1,
    dest="limit_story",
    metavar="LIMIT",
  )
  parser.add_argument(
    "-v",
    "--verbose",
    help="FFmpeg output verbosity.",
    action="store_true",
    default=False,
    dest="verbose",
  )
  parser.add_argument(
    "--sleep-interval",
    help="Sleep between downloads in seconds. (Default: 1s)",
    type=int,
    default=1,
    dest="sleep_interval",
    metavar="INTERVAL",
  )
  parser.add_argument(
    "--image-duration",
    help="Set duration for image in seconds. (Default: 1s)",
    type=int,
    default=1,
    dest="loop_duration_image",
    metavar="DURATION",
  )
  parser.add_argument(
    "--version", action="version", version=f"%(prog)s {version('concat_stories')}"
  )

  if not is_ffmpeg_installed():
    logger.error("FFmpeg binary not found. Please install FFmpeg or add it to PATH.")
    logger.error("Download FFmpeg from https://ffmpeg.org/download.html")
    sys.exit(1)

  if len(sys.argv) == 1:
    parser.print_help()
    sys.exit(1)

  args = parser.parse_args()

  try:
    sorted_stories = SnapchatDL(
      sleep_interval=args.sleep_interval, limit_story=args.limit_story
    ).download(args.username)

    if args.wait:
      logger.info("Press Enter to continue after deleting unwanted stories.")
      input("Press Enter to continue...")

    # We know sorted_stories is not empty
    dir_path = sorted_stories[0].rsplit("/", 1)[0]

    folder_name = dir_path.split("/")[-1]
    output_name = args.output or folder_name

    concat_stories = ConcatStories(
      sorted_stories,
      output_name,
      loop_duration_image=args.loop_duration_image,
      is_quiet=not args.verbose,
    )
    concat_stories.concat()

    if args.delete:
      shutil.rmtree(dir_path)
      logger.info(f"Stories deleted from {dir_path}")
  except KeyboardInterrupt:
    logger.info("Process interrupted by user.")
  except Exception as e:
    logger.error(f"An error occurred: {e}")
    sys.exit(1)


if __name__ == "__main__":
  sys.exit(main())



import concurrent.futures
from datetime import datetime, timezone
import json
import os
import re
import time
import requests
from loguru import logger


class NoStoriesFound(Exception):
  """No stories found."""

  pass


class APIResponseError(Exception):
  """Invalid API Response"""

  pass


MEDIA_TYPE = ["jpg", "mp4"]


class SnapchatDL:
  """Interact with Snapchat API to download story."""

  def __init__(
    self, directory_prefix="stories", max_workers=2, limit_story=-1, sleep_interval=1
  ):
    self.directory_prefix = os.path.abspath(os.path.normpath(directory_prefix))
    self.max_workers = max_workers
    self.limit_story = limit_story
    self.sleep_interval = sleep_interval
    self.endpoint_web = "https://www.snapchat.com/add/{}/"
    self.regexp_web_json = (
      r'<script\s*id="__NEXT_DATA__"\s*type="application\/json">([^<]+)<\/script>'
    )
    self.reaponse_ok = requests.codes.get("ok")

  def _api_response(self, username: str) -> str:
    web_url = self.endpoint_web.format(username)
    return requests.get(
      web_url,
      headers={
        "User-Agent": "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/131.0.0.0 Safari/537.36"
      },
    ).text

  def _web_fetch_story(self, username: str) -> list[dict]:
    """Download user stories from Web.

    Args:
        username (str): Snapchat `username`

    Raises:
        APIResponseError: API Error

    Returns:
        list: list of stories
    """
    response = self._api_response(username)
    response_json_raw = re.findall(self.regexp_web_json, response)

    try:
      response_json = json.loads(response_json_raw[0])

      def util_web_story(content: dict):
        if "story" in content["props"]["pageProps"]:
          return content["props"]["pageProps"]["story"]["snapList"]
        return list()

      stories = util_web_story(response_json)
      return stories

    except (IndexError, KeyError, ValueError):
      raise APIResponseError

  def _download_url(self, url: str, dest: str, sleep_interval: int):
    """Download URL to destionation path.

    Args:
        url (str): url to download
        dest (str): absolute path to destination
        sleep_interval (int): sleep interval

    Raises:
        response.raise_for_status: if response is 4** or 50*
        FileExistsError: if file is already downloaded
    """
    if len(os.path.dirname(dest)) > 0:
      os.makedirs(os.path.dirname(dest), exist_ok=True)

    """Rate limiting."""
    time.sleep(sleep_interval)

    try:
      response = requests.get(url, stream=True, timeout=10)
    except requests.exceptions.ConnectTimeout:
      response = requests.get(url, stream=True, timeout=10)

    if response.status_code != requests.codes.get("ok"):
      response.raise_for_status()

    if os.path.isfile(dest) and os.path.getsize(dest) == response.headers.get(
      "content-length"
    ):
      raise FileExistsError

    if os.path.isfile(dest) and os.path.getsize(dest) == 0:
      os.remove(dest)
    try:
      with open(dest, "xb") as handle:
        try:
          for data in response.iter_content(chunk_size=4194304):
            handle.write(data)
          handle.close()
        except requests.exceptions.RequestException as e:
          logger.error(e)
    except FileExistsError:
      pass

  def download(self, username: str) -> list[str]:
    """Download Snapchat Story for `username`.

    Args:
        username (str): Snapchat `username`

    Returns:
        list: list of downloaded sorted stories by timestamp
    """
    stories = self._web_fetch_story(username)

    if len(stories) == 0:
      logger.info(f"\033[91m{username}\033[0m has no stories")
      raise NoStoriesFound

    if self.limit_story > -1:
      stories = stories[0 : self.limit_story]

    logger.info(f"[+] {username} has {len(stories)} stories")

    executor = concurrent.futures.ThreadPoolExecutor(max_workers=self.max_workers)

    now_date = datetime.now().strftime("%Y-%m-%d_%H-%M-%S")
    folder_name = f"{username}_{now_date}"
    dir_path = os.path.join(self.directory_prefix, folder_name)

    sorted_stories = []

    try:
      for media in stories:
        snap_id = media["snapId"]["value"]
        media_url = media["snapUrls"]["mediaUrl"]
        media_type = media["snapMediaType"]
        timestamp = int(media["timestampInSec"]["value"])

        filename = datetime.fromtimestamp(timestamp, timezone.utc).strftime(
          f"%Y-%m-%d_%H-%M-%S_{username}_{snap_id}.{MEDIA_TYPE[media_type]}"
        )

        media_output = os.path.join(dir_path, filename)
        executor.submit(
          self._download_url, media_url, media_output, self.sleep_interval
        )

        sorted_stories.append((timestamp, media_output))

    except KeyboardInterrupt:
      executor.shutdown(wait=False)

    # wait for all downloads to finish
    executor.shutdown(wait=True)
    logger.info(f"[✔] {username} stories downloaded")

    sorted_stories.sort(key=lambda x: x[0])
    sorted_stories = [story[1] for story in sorted_stories]
    return sorted_stories
